
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;

import com.sun.jna.Library;
import com.sun.jna.Native;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.win32.StdCallFunctionMapper;

import lib.DFCom;

/**
 * Klasse in der Funktionen geschrieben wurden zur einfachen Nutzung einiger DLL Funktionen  (DFComDLL.dll).
 * Nicht alle Funktionen wurden geprft und benutzt und knnen Fehlerhaft sein.
 * @author Harald Kranz
 *
 */
public class DFController {
	
	/**
	 * Intern zur Benutzung der DLL Methoden aus dem Interface DFCom.
	 */
	DFCom dfCom;
    
	/**
	 * Liste der ausgelesenen Datensatzbeschreibungen
	 */
	private List<DFRecordDescription> allRecordDescription = new ArrayList<DFRecordDescription>();
	
	/**
	 * Liste der ausgelesenen Listenbeschreibungen
	 */
	private List<DFRecordDescription> allListDescription = new ArrayList<DFRecordDescription>();
	
	
	/**
	 *  Konstante fr TCPIP Verbindungstyp anderer Zugriff mglich siehe DLL Beschreibung.
     */
    public static final int TCPIP = 3;
	/**
	 * Kanalnalnummer 
	 */
	private int channelID = 1;
	/**
	 * Busnummer des Komunikationsteilnehmer
	 */
	private int deviceID = 1;
	/**
	 * Komunikationstyp 3 = TCP IP
	 */
	private int communicationTyp = TCPIP;
	
	/**
	 * Kommunicationsdaten bei TCP IP = IP-Adresse
	 */
	private String comunicationString = "";
	
	
	/**
	 *  Kumunikationsdaten bei TCP IP = Portnummer (Standard = 8000)
	 */
	private int comuncationValue = 8000;
	
	/**
	 * Kumunikationstimeout in Millisekunden Standard = 5000 ms ;
	 */
	
	private int comunicationTimeOut = 5000;
	
	
	
	/**
	 * enthlt den aktuellen Verbindungsstatus 
	 */
	private boolean connected = false;
	
	
	/**
	 * Konstruktor mit JNA ZUgriff auf die DLL (DFComDLL)
	 * mit Kanalnummer und Gertenummer die fr die Kommunikation bentigt wird.
	 * @param dfComDLL = Pfad zur DFComDLL
	 * @param channelID = Kanalnummer 
	 * @param deviceID = Gertenummer
	 */
	public DFController(String dfComDLL,int channelID, int deviceID) {
		HashMap options = new HashMap();
		StdCallFunctionMapper mapper = new StdCallFunctionMapper();
		options.put(Library.OPTION_FUNCTION_MAPPER,mapper);    
	    this.dfCom = (DFCom) Native.loadLibrary(dfComDLL, DFCom.class, options);
		this.channelID = channelID;
		this.deviceID = deviceID;     
	}
	
	
	/**
	 * Kommunkation herstellen alle Parameter sollten richtig eingestellt werden in der Klasse. 
	 * @return Error = 0 Fehler beim Initialisieren, 1 = erfolgreich.
	 */
	public int comOpen()
	{
		return this.dfCom.DFCComOpenIV(this.channelID, this.deviceID, this.communicationTyp, this.comunicationString, this.comuncationValue,this.comunicationTimeOut);
	}
	
	/**
	 * Kommunkation herstellen ber TCP IP nur ipAdresse mit eingestellter Portnummer ohne nderung wird 8000 benutzt;
	 * @return Error = 0 Fehler beim Initialisieren, 1 = erfolgreich.
	 */	
	public int comOpen(String ipAdresse)
	{
		int ret = 0;
		this.communicationTyp = this.TCPIP;
		this.comunicationString = ipAdresse;
		this.comuncationValue = 8000;
		 ret = this.dfCom.DFCComOpenIV(this.channelID, this.deviceID, this.communicationTyp, this.comunicationString, this.comuncationValue,this.comunicationTimeOut);		
		 // setzen des Verbindungsstaus
		 if (ret == 0) 
			this.connected = false;
		 else 
			 this.connected = true; 
	     return ret;
	}	

	/**
	 * Kommunkation ber TCP IP mit Portnummer;
	 * @param ipAdresse als String 
	 * @param portNumber als Integer
	 * @return Error = 0 Fehler beim Initialisieren, 1 = erfolgreich.
	 */
	public int comOpen(String ipAdresse,int portNumber)
	{
		int ret = 0;
		this.communicationTyp = this.TCPIP;
		this.comunicationString = ipAdresse;
		this.comuncationValue = portNumber;
		ret =  this.dfCom.DFCComOpenIV(this.channelID, this.deviceID, this.communicationTyp, this.comunicationString, this.comuncationValue,this.comunicationTimeOut);
		// Setzen des Verbindungsstatus
		if (ret == 0) 
			this.connected = false;  
		else
			this.connected = true;
		return ret;
	}
	
	/**
	 * Offene Kommuniktion mit Einstellung des KomunikationsTyp siehe DLL Beschreibung.
	 * @param communicationTyp mglich ist hier 1 = RS232 direkt oder ber Umsetzter (RS232 <-> RS485) 
												2 Funk ber den Umsetzer (RS232 <-> BIM2) ohne eigenen Kontroller. 
												3 TCP / IP - Verbindung. 
	 * @param comunicationString    			fr Typ 1, 2 fr die Schnittstelle. 
												fr Typ 3  mit IP - Adresse oder Hostname. 

	 * @param comunicationValue                 fr Typ 1, 2 bertragungsrate der Schnittstelle in Millisekunden. (Standard: 38400bd) 
												fr Typ 3 Portnummer des TCP / IP Anschlusses. (Standard: 8000) 
	 * @param comunicationTimeOut				fr Typ 1, 2 Timeout fr Lesen von Schnittstelle in Millisekunden. (Standard: 800ms)
													Bei Modem min. 1500ms empfohlen. 
												3 Timeout fr Lesen von Socket in Millisekunden. (Standard: 5000ms) 

	 * @return Error = 0 Fehler beim Initialisieren, 1 = erfolgreich.
	 */
	public int comOpen(int communicationTyp,String comunicationString,int comunicationValue, int comunicationTimeOut )
	{
		int ret = 0;
		this.communicationTyp = communicationTyp;
		this.comunicationString = comunicationString;
		this.comuncationValue = comunicationValue;
		this.comunicationTimeOut = comunicationTimeOut;
		ret = this.dfCom.DFCComOpenIV(this.channelID, this.deviceID, this.communicationTyp, this.comunicationString, this.comuncationValue,this.comunicationTimeOut);
		// Setzen des Verbindungsstatus
		if (ret == 0 )
		   this.connected = false;
		else 
		   this.connected = true;
		return ret;
	}
	
	/**
	 * Verbindung zum Gert wird beendet.
	 */
	public void comClose()
	{
		this.dfCom.DFCComClose(this.channelID);
		// Setzen des Verbindungsstatus
		this.connected = false;
	}
	
   /**
   * Anzeige eines Text auf dem Gert
   * @param message Gesammte Zeichenkette die Trennung erfolgt mit \r
   * @param delayTime  wie lang die Message angezeigt werden soll in Sekunden 
   * @param messageBeep 0 - Off, 1 = OK, 2 = Error weitere siehe DLL Beschreibung
   * @return  0 = Fehler beim Senden der Message, 1 = Message erfolgreich gesendet
   * @throws DFError Wenn keine Verbindung zum Gert erstellt wurde. 
   */
	public int sendMessage(String message,int delayTime,int messageBeep) throws DFError
	{
		if (this.connected) 
		   return this.dfCom.DFCComSendMessage(this.channelID, this.deviceID, delayTime, messageBeep,0 , message, message.length());
		else
		{
			throw new DFError("Fehler Sie mssen erst eine Verbindung zum Gert herstellen.");
		}
	}
	
	/**
	 * Anzeige eines Text auf dem Gert 
	 * @param message Gesammte Zeichenkette die Trennung erfolgt mit \r
	 * @param delayTime wie lang die Message angezeigt werden soll in Sekunden 
	 * @return  0 = Fehler beim Senden der Message, 1 = Message erfolgreich gesendet
	 * @throws DFError Wenn keine Verbindung zum Gert erstellt wurde. 
	 */
	public int sendMessage(String message,int delayTime) throws DFError
	{
		if (this.connected)
		   return this.dfCom.DFCComSendMessage(this.channelID, this.deviceID, delayTime, 0,0, message, message.length());
		else 
			throw new DFError("Fehler Sie mssen erst eine Verbindung zum Gert herstellen.");
	}
	
	/**
	 * Anzeige eines Text auf dem Gert 
	 * @param headerLine1  Erste Zeile der berschrift ohne Zeilenumbruch 
	 * @param haederLine2  Zweite Zeile der berschrift ohne Zeilenumbruch
	 * @param boxHeader   Boxenberschrift eine Zeile ohne Zeilenumbruch  
	 * @param BoxText     Boxentext kann mehrzeilig sein Zeilenumbruch mit \r
	 * @param delayTime   wie lang die Message angezeigt werden soll in Sekunden 
	 * @param messageBeep 0 - Off, 1 = OK, 2 = Error weitere siehe DLL Beschreibung
	 * @return 0 = Fehler beim Senden der Message, 1 = Message erfolgreich gesendet
	 * @throws DFError Wenn keine Verbindung zum Gert erstellt wurde. 
	 */
	public int sendMessage(String headerLine1,String headerLine2, String boxHeader, String BoxText,int delayTime,int messageBeep) throws DFError
	{		
		String message = headerLine1+"\r"+headerLine2+"\r"+boxHeader+"\r"+BoxText;		
		if (this.connected)
		   return this.dfCom.DFCComSendMessage(this.channelID, this.deviceID, delayTime, messageBeep , 1, message, message.length());
		else 
			throw new DFError("Fehler Sie mssen erst eine Verbindung zum Gert herstellen.");
	}
	
	/**
	 * Auslesen aller Datensatzbeschreibungen aus dem Gert zu dem eine Verbindung aufgebaut wurde.
	 * @return Liste der Datensatzbeschreibungen 
	 * @throws DFError Keine VErbindung zum Gert vorhanden.
	 */
	public List<DFRecordDescription> readRecordDescription() throws DFError
	{
		allRecordDescription.clear();
		if (this.connected) 
		{
			IntByReference err = new IntByReference();
			// Laden der Datensatzbeschreibungen einleiten
			this.dfCom.DFCLoadDatensatzbeschreibung(this.channelID, this.deviceID, err);
			// Feststellen der Anzahl Datensatzbeschreibungen 
			int recordDescriptionCount = this.dfCom.DFCDatBCnt(this.channelID);
			
			byte[] dataName = new byte[17];
			byte[] fieldName = new byte[17];
			IntByReference refFieldCount = new IntByReference();
			IntByReference refFieldType= new IntByReference();
			IntByReference refFieldLength = new IntByReference();			
			// Schleife durch alle Datensatzbeschreibungen 
			for ( int i = 0; i < recordDescriptionCount; i++)
			{
				// Auslesen der Datensatzbeschreibung 
				this.dfCom.DFCDatBDatensatz(this.channelID, i, dataName, refFieldCount);
				DFRecordDescription dfRecordDescription = new DFRecordDescription();
				dfRecordDescription.setRecordNumber(i);
				dfRecordDescription.setRecordName(new String(dataName).trim());
				List<DFFieldDescription> allFieldDescriptionForRecord  = new ArrayList<DFFieldDescription>();
				// Schleife durch alle Feldbeschreibungen der Datensatzbeschreibung 
				for ( int j = 0; j < refFieldCount.getValue(); j++)
				{
					// Auslesen der Feldbeschreibungen 
					DFFieldDescription dfFieldDescription = new DFFieldDescription();				
					this.dfCom.DFCDatBFeld(this.channelID, i, j, fieldName, refFieldType, refFieldLength);
					dfFieldDescription.setFieldNumber(j);
					dfFieldDescription.setFieldName(new String(fieldName).trim());
					dfFieldDescription.setFieldType(refFieldType.getValue());
					dfFieldDescription.setFieldLength(refFieldLength.getValue());
					allFieldDescriptionForRecord.add(dfFieldDescription);
				}
				dfRecordDescription.setDfFields(allFieldDescriptionForRecord);
				allRecordDescription.add(dfRecordDescription);
			}
			return allRecordDescription;
		} else 
			throw new DFError("Fehler Sie mssen erst eine Verbindung zum Gert herstellen.");
	}
	
	
	/**
	 * Auslesen aller Listenbeschreibungen aus dem Gert zu dem eine Verbindung aufgebaut wurde.
	 * @return Liste der Datensatzbeschreibungen 
	 * @throws DFError Keine VErbindung zum Gert vorhanden.
	 */
	public List<DFRecordDescription> readListDescription() throws DFError
	{
		allListDescription.clear();
		if (this.connected) 
		{
			IntByReference err = new IntByReference();
			// Laden der Listenbeschreibung einleiten
			this.dfCom.DFCLoadListenbeschreibung(this.channelID, this.deviceID, err);
			// Feststellen der Anzahl Listenbeschreibungen 
			int recordDescriptionCount = this.dfCom.DFCListBCnt(this.channelID);
			
			byte[] listName = new byte[17];
			byte[] fieldName = new byte[17];
			IntByReference refFieldCount = new IntByReference();
			IntByReference refFieldType= new IntByReference();
			IntByReference refFieldLength = new IntByReference();
			IntByReference refFieldVar = new IntByReference();
			// Schleife durch alle Datensatzbeschreibungen 
			for ( int i = 0; i < recordDescriptionCount; i++)
			{
				// Auslesen der Listenbeschreibungen 
				this.dfCom.DFCListBDatensatz(this.channelID, i, listName, refFieldCount,refFieldVar);
				DFRecordDescription dfListDescription = new DFRecordDescription();
				dfListDescription.setRecordNumber(i);
				dfListDescription.setRecordName(new String(listName).trim());
				List<DFFieldDescription> allFieldDescriptionForList  = new ArrayList<DFFieldDescription>();
				// Schleife durch alle Feldbeschreibungen der Datensatzbeschreibung 
				for ( int j = 0; j < refFieldCount.getValue(); j++)
				{
					// Auslesen der Feldbeschreibungen 
					DFFieldDescription dfFieldDescription = new DFFieldDescription();				
					this.dfCom.DFCListBFeld(this.channelID, i, j, fieldName, refFieldType, refFieldLength);
					dfFieldDescription.setFieldNumber(j);
					dfFieldDescription.setFieldName(new String(fieldName).trim());
					dfFieldDescription.setFieldType(refFieldType.getValue());
					dfFieldDescription.setFieldLength(refFieldLength.getValue());
					allFieldDescriptionForList.add(dfFieldDescription);
				}
				dfListDescription.setDfFields(allFieldDescriptionForList);
				allListDescription.add(dfListDescription);
			}
			return allListDescription;
		} else 
			throw new DFError("Fehler Sie mssen erst eine Verbindung zum Gert herstellen.");
	}
	
	
	/**
	 * Zum lesen eines Datensatz als DFRecord
	 * @return null oder einen Datensatz als String
	 * @throws DFError Erzeugt einen Fehler wenn Datensatz nicht gelesen werden kann.
	 * @throws DFError Erzeugt einen Fehler wenn keine Verbindung zum Gert vorhanden ist.
	 * @throws DFError Erzeugt einen Fehler wenn das Quttieren des Datensatz nicht geht.	
	 */
	public DFRecord readRecord() throws DFError
	{
		if (this.connected) 
		{
		   // Datensatzbeschreibungen vom Gert lesen	
		   this.readRecordDescription();
		   DFRecord rec = new DFRecord();
		   byte[] buf = new byte[255];
		   IntByReference length = new IntByReference(buf.length);
		   IntByReference err = new IntByReference();
		   int retReadRecord = retReadRecord = this.dfCom.DFCReadRecord(this.channelID, this.deviceID, buf, length, err);
		   if (retReadRecord > 0)	   
		   {
			  rec.setTyp(Integer.parseInt(String.valueOf(buf[0]))); 
			  rec.setByteValue(buf);
			  for (int i = 0; i< this.allRecordDescription.size();i++)
			  {
				  if (allRecordDescription.get(i).getRecordNumber()== rec.getTyp())
				  {
					  rec.setStringValue(formatRecord(rec, allRecordDescription.get(i)));
				  }
			  }			  
			  if (this.dfCom.DFCQuitRecord(this.channelID, this.deviceID, err) == -1 )
			  	throw new DFError("Fehler aufgetreten beim Quttieren des Datensatz.");
			  
			  return rec;
		   }
		   else 
		   {
			   if (retReadRecord == 0)
				  return null; 	
			   else 
				  throw new DFError("Fehler aufgetreten beim lesen des Datensatz.");

		   }
		}
		else
			throw new DFError("Fehler Sie mssen erst eine Verbindung zum Gert herstellen.");
			
	}
	
	/**
	 * Alle Datenstze auslesen
	 * @return List<DFRecord> Liste der Datenbstze
	 * @throws DFError werden weitergeleitet aus readRecord
	 */
	public List<DFRecord> readAllRecords() throws DFError
	{
		List<DFRecord> listRec = new ArrayList<DFRecord>();
		DFRecord rec;
		while ((rec = readRecord()) != null){
		   listRec.add(rec);	
		}
		return listRec;
	}
	
	/**
	 * Formatiert den Record entsprechend der bergebenen Datensatzbeschreibung  
	 * @param record DFRecord Datesatz der Formatiert werden soll 
	 * @param desc  DFRecordDescription Formatvorlage fr den Datensatz 
	 * @return String Formatierten Datensatz 
	 * @throws DFError 
	 */
	private String formatRecord(DFRecord record ,DFRecordDescription desc) throws DFError
	{
		int pos = 1; // Position im ByteArray des Datensatz 
		List<DFFieldDescription> lFieldDesc =  desc.getDfFields(); // Feldbeschreibungen
	    byte[] buf = record.getByteValue(); // Datensatz als ByteArray
	    StringBuffer strBuf = new StringBuffer(); // zum zusammenstellen des Rckgabewertes
		
	    // Schleife durch alle Feldbeschreibungen 
	    for (int i = 0;i < lFieldDesc.size(); i++)
		{
		   DFFieldDescription fieldDesc = lFieldDesc.get(i);
		   // Feldtypenspezifische Formatierung des String 
	       switch (fieldDesc.getFieldType()) {
		   case 1: // DF_LONG 			
			  throw new DFError("Fehler ein Datentyp wird nicht untersttzt.");
		   case 2: // DF_DATETIME 
			  int[] dateTimeArr = new int[6];
			  dateTimeArr[0] = Integer.parseInt(String.valueOf(buf[pos]));
			  dateTimeArr[0] = dateTimeArr[0]* 100 + Integer.parseInt(String.valueOf(buf[pos+1]));
		      dateTimeArr[1] =  Integer.parseInt(String.valueOf(buf[pos+2]));
		      dateTimeArr[2] =  Integer.parseInt(String.valueOf(buf[pos+3]));
		      dateTimeArr[3] =  Integer.parseInt(String.valueOf(buf[pos+4]));
		      dateTimeArr[4] =  Integer.parseInt(String.valueOf(buf[pos+5]));
		      dateTimeArr[5] =  Integer.parseInt(String.valueOf(buf[pos+6]));
		      Date dateTime =new GregorianCalendar(dateTimeArr[0],dateTimeArr[1],dateTimeArr[2],dateTimeArr[3],dateTimeArr[4],dateTimeArr[5]).getTime();
		      SimpleDateFormat df = new SimpleDateFormat( "dd.MM.yyyy HH:mm:ss" );
		      strBuf.append(df.format(dateTime));
		      strBuf.trimToSize();
		      strBuf.append("\t");
		      break;
		   case 3: // DF_ZIFFER
		      strBuf.append(new String(buf,pos ,fieldDesc.getFieldLength()).trim());
		      strBuf.append("\t");
		      break;
		   case 4: // DF_ALPHANUM
			  strBuf.append(new String(buf,pos ,fieldDesc.getFieldLength()).trim());
		      strBuf.append("\t");
			  break;
		   case 5: // DF_DATE
				throw new DFError("Fehler ein Datentyp wird nicht untersttzt.");
		   case 6: // DF_TIME 
				throw new DFError("Fehler ein Datentyp wird nicht untersttzt.");
		   default:
				throw new DFError("Fehler ein Datentyp wird nicht untersttzt.");
	       }
	       // Position verschieben
		   pos = pos+fieldDesc.getFieldLength();
		}
		return strBuf.toString();
	}

	/**
	 * List die Listen aus einem Verzeichnis aus.
	 * Listen werden nur erkannt wenn sie die gleiche Bezeichnung wie in der Listenbeschreibung ".txt" haben.
	 * @param pathToListFiles Pfad in dem die Listen liegen sollen.
	 * @return List<DFListRecord> Liste der Listendaten
	 * @throws IOException wenn die Datei nicht geffnet werden kann.
	 * @throws DFError durchleiten der DLL Fehler
	 */
	public List<DFListRecord> readListData(String pathToListFiles) throws IOException, DFError
	{
		readListDescription();
		List<DFListRecord> listRecords = new ArrayList<DFListRecord>();
		// Schleife durch alle Listenbeschreibungen
		for (int i = 0; i < allListDescription.size();i++)
		{
			// Datei mit Listendaten finden
			List<String> fileLines = readFile(pathToListFiles+allListDescription.get(i).getRecordName()+".txt");
			// Schleife durch alle Zeilen der Datei.
			for (int j= 0; j< fileLines.size();j++)
			{
				DFListRecord listRecord = new DFListRecord();
				listRecord.setStringValue(fileLines.get(j));
				listRecord.setFileName(allListDescription.get(i).getRecordName());
				listRecord.setListNumber(allListDescription.get(i).getRecordNumber());
				listRecord.setByteValue(formatListRecord(listRecord,allListDescription.get(i)));
				listRecords.add(listRecord);
			}
		}
		return listRecords;
	}
	
	/**
	 * Liest eine Datei in eine String Liste.
	 * @param file Pfad und Dateiname
	 * @return List <String> pro Zeile ein String  
	 * @throws IOException wenn die Datei gelesen weden kann.
	 */
	private List<String> readFile(String file) throws IOException 
	{
		List<String> listLines =new ArrayList<String>();
		FileReader fr = null;
		fr = new FileReader(file);
    	String zeile = null;
	    BufferedReader br = new BufferedReader(fr);
	    while ((zeile = br.readLine()) != null)
			listLines.add(zeile);	
		return listLines;	
	}
	
	/**
	 * Formatiert die bergebene Listdatensatz  mit hilfe der Datenbeschreibung der Liste. 
	 * @param listRecord Listendatensatz
	 * @param listDesc Listenbeschreibung zum Datensatz 
	 * @return Listendatensatz als formatiertes byte[]
	 * @throws DFError weiterleitung mglicher Fehler
	 */
	private byte[] formatListRecord(DFListRecord listRecord,DFRecordDescription listDesc) throws DFError
	{
		int posTemp = 0; 
		int pos = 0;
		
		byte[] tempBuf = (listRecord.getStringValue()+"\t").getBytes();
		StringBuffer tempStrBuf = new StringBuffer(listRecord.getStringValue());
		List<DFFieldDescription> listFieldDesc = new ArrayList<DFFieldDescription>(); 
		listFieldDesc = listDesc.getDfFields();

		int maxBufLength = 0;
		for ( int i = 0; i < listFieldDesc.size();i++)
			maxBufLength += listFieldDesc.get(i).getFieldLength();

		byte[] formatBuf = new byte[maxBufLength];
		for (int i = 0; i < formatBuf.length;i++)
		   formatBuf[i] = 0;	
		
		for ( int i = 0; i < listFieldDesc.size();i++)
		{
			int fieldType = listFieldDesc.get(i).getFieldType();
			int z;
			switch (fieldType) {
			case 1: // DF_DOUBLE	
				throw new DFError("Fehler ein Datentyp wird nicht untersttzt.");
			case 2:// DF_DATETIME
				throw new DFError("Fehler ein Datentyp wird nicht untersttzt.");
			case 3:// DF_ZIFFER
				z = 0;
				while(tempBuf[posTemp] != 9 && posTemp < tempBuf.length) // Trenner ist Tabulator das muss geprft werden
				{
					if (Character.isDigit(tempStrBuf.charAt(posTemp)))
					{
						formatBuf[pos+z] = tempBuf[posTemp];
					} else {
						throw new DFError("Fehler keine gltige Zahl im Listenfeld.");
					}
					z++;
					posTemp++;
				}
				posTemp++; // +  Tab uebergehen
				break;
			case 4:	// DF_ALPHANUM	
				z = 0;
				while(tempBuf[posTemp] != 9 && posTemp < tempBuf.length) // Trenner ist Tabulator das muss geprft werden
				{
  				    formatBuf[pos+z] = tempBuf[posTemp];
					z++;
					posTemp++;
				}
				posTemp++; // + tab uebergehen
				break;
			case 5:	//DF_DATE			
				throw new DFError("Fehler ein Datentyp wird nicht untersttzt.");
			case 6:	//DF_TIME			
				throw new DFError("Fehler ein Datentyp wird nicht untersttzt.");
			default:
				throw new DFError("Fehler ein Datentyp wird nicht untersttzt.");
			}
			pos = pos+listFieldDesc.get(i).getFieldLength();
		}
		return formatBuf;
		
	}
	
	/**
	 * bertragen der Listendaten auf das verbundene Gert. 
	 * Dazu muss das zugehrige Setup immer mit bertragen werden. 
	 * @param setupFile Pfad und Dateiname des zu verwendenden Setup
	 * @param pathToListFiles Pfad zu der Listendateien 
	 * @throws IOException wenn eine Datei nicht bearbeitet werden kann.
	 * @throws DFError bergabe mglicher Fehler 
	 */
	public void writeLists(String setupFile,String pathToListFiles) throws IOException, DFError
	{
		IntByReference err = new IntByReference();
		if (this.connected)
		{
			// Lesen der Listendaten
			List<DFListRecord> listRecords = readListData(pathToListFiles);
					
			if(listRecords.size()>0)
			{
			    dfCom.DFCClrListenBuffer(this.channelID);
			    // hier wird gespeichert wie viel Zeilen zu jeder Liste vorhanden sind
			    // listLineNumbers[0] ist dann die erste Liste  
			    int[] listLineNumbers = new int[20]; // Anzahl der mglichen Listen ist 20,
		    	int[] listLineSize = new int[20]; 
			    int listNumber = 0;
		    	int lineNumber = 0;
		    	// auslesen wie viel Zeilen zu jeder Liste vorhanden sind
			    for (int i = 0;i < listRecords.size(); i++)
			    {
			    	if (listRecords.get(i).getListNumber() == listNumber)
			    	{
			    		lineNumber++;
			    		listLineNumbers[listNumber] = lineNumber; 
			    		listLineSize[listNumber] = listRecords.get(i).getByteValue().length;
			    	} else {
			    		lineNumber = 1;
			    		listNumber++;
			    	}
			    }
			    
			    int z = 0;
			    int nextPos = 0;
			    // Schleife durch Anzahl der unterschiedlichen Listen  
			    while (listLineNumbers[z] > 0)
			    {
				    int posNextLine = 0;
			    	byte[] buf = new byte[listLineNumbers[z]*listLineSize[z]];
			    	for (int i = nextPos; i < nextPos+listLineNumbers[z]; i++)
			    	{
			    		for (int j = 0; j < listRecords.get(i).getByteValue().length ;j++)
			    		{
			    			buf[j+posNextLine*listLineSize[z]] = listRecords.get(i).getByteValue()[j];
			    		}
			    		posNextLine++;
			    	}
			    	// Liste in der DLL erstellen
					if (dfCom.DFCMakeListe(this.channelID,z,listLineNumbers[z], buf.length, buf, 0)==0)
						 throw new DFError("Fehler Daten konnten nicht an die DLL bergeben werden.");						
			    	
			    	nextPos = nextPos+listLineNumbers[z]; 
			        z++;
			    }
			    
			 } else {
				 throw new DFError("Fehler keine Listendaten zum bertragen vorhanden.");
			 }
			 
			 // Setupdatei bertragen 
			 if (dfCom.DFCSetupLaden(this.channelID, this.deviceID,setupFile, err) == 0)
			 { 
				  System.out.println(err.getValue());
				 throw new DFError("Fehler beim laden des Setup.");
			 }
			
			 // Listen bertragen 
			 if (dfCom.DFCLoadListen(this.channelID, this.deviceID,err)==0)
				 throw new DFError("Fehler beim bertragen der Listen auf das Gert.");
	
		}
	}
	
	/**
	 * Statusanzeige Verbindungsaufbau
	 * @return Connection vorhanden
	 */
	public boolean isConnected() {
		return connected;
	}
	

	
}
