#include "../df_lists.h"
#include "../../DatafoxLibraryIV.h"

DF_API_EXTERN int DF_API_CALL DFCOpenTable( int connectionId, int deviceAddress, char *name, int options, int *tableHandle, char *reserved, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (name == NULL) || (tableHandle == NULL) || (reserved == NULL) || (errorNumber == NULL) ||
        (options != 0) || ((reserved != NULL) && strlen(reserved)) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t%i\t%s\t%i\t(%i)", (name == NULL) ? "" : name, options, (tableHandle == NULL) ? 0 : *tableHandle, (reserved == NULL) ? "" : reserved, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%s\t%i\t%i\t%s\t%i\t(%i)", name, options, *tableHandle, reserved, *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->tableOpen(deviceAddress, name, options, (int&)*tableHandle, reserved);
    if (iRes == IV_ERROR_SUCCESS)
    { // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    { // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
            case 2:
                *errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 28:
                *errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%i\t%i\t%s\t%i\t(%i)", name, options, *tableHandle, reserved, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCCloseTable( int connectionId, int deviceAddress, int tableHandle, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->tableClose(deviceAddress, tableHandle);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                *errorNumber = IV_ERROR_HANDLE;
                break;

            case 28:
                *errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCSetFilter( int connectionId, int deviceAddress, int tableHandle, char *value, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (value == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t(%i)", (value == NULL) ? "" : value, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%s\t%i\t(%i)", value, *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->tableSetFilter(deviceAddress, tableHandle, value);
    if (iRes == IV_ERROR_SUCCESS)
    { 
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                *errorNumber = IV_ERROR_HANDLE;
                break;

            case 3:
                *errorNumber = IV_ERROR_NULL_RESULT;
                break;

            case 4:
                *errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 28:
                *errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetFilter( int connectionId, int deviceAddress, int tableHandle, char *value, int *valueLength, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (value == NULL) || (valueLength == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t%i\t(%i)", (value == NULL) ? "" : value, (valueLength == NULL) ? 0 : *valueLength, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%s\t%i\t%i\t(%i)", value, *valueLength, *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->tableGetFilter(deviceAddress, tableHandle, value, (short&)*valueLength);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                *errorNumber = IV_ERROR_HANDLE;
                break;

            case 28:
                *errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%i\t%i\t(%i)", value, *valueLength, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCClearFilter( int connectionId, int deviceAddress, int tableHandle, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->tableClearFilter(deviceAddress, tableHandle);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                *errorNumber = IV_ERROR_HANDLE;
                break;

            case 28:
                *errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCSkip( int connectionId, int deviceAddress, int tableHandle, int offset, int origin, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle, offset, origin);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) || ((origin != 0) && (origin != 1) && (origin != 2)) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->tableSkip(deviceAddress, tableHandle, offset, origin);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                *errorNumber = IV_ERROR_HANDLE;
                break;

            case 3:
                *errorNumber = IV_ERROR_BOF;
                break;

            case 4:
                *errorNumber = IV_ERROR_EOF;
                break;

            case 28:
                *errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCSetField( int connectionId, int deviceAddress, int tableHandle, char *name, char *value, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (name == NULL) || (value == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%s\t%i\t(%i)", (name == NULL) ? "" : name, (value == NULL) ? "" : value, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%s\t%s\t%i\t(%i)", name, value, *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->tableSetField(deviceAddress, tableHandle, name, value);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                *errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                *errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 3:
                *errorNumber = IV_ERROR_VALUE;
                break;

            case 28:
                *errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%s\t%i\t(%i)", name, value, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetField( int connectionId, int deviceAddress, int tableHandle, char *name, char *value, int *valueLength, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (name == NULL) || (value == NULL) || (valueLength == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%s\t%i\t%i\t(%i)", (name == NULL) ? "" : name, (value == NULL) ? "" : value, (valueLength == NULL) ? 0 : *valueLength, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%s\t%s\t%i\t%i\t(%i)", name, value, *valueLength, *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->tableGetField(deviceAddress, tableHandle, name, value, (short&)*valueLength);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                *errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                *errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 28:
                *errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%s\t%i\t%i\t(%i)", name, value, *valueLength, *errorNumber, iRes);

    return iRes;
}
