#include "../df_setup_records.h"
#include "../../DatafoxLibraryIV.h"

DF_API_EXTERN int DF_API_CALL DFCLoadDatensatzbeschreibung( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->getTableDescription(deviceAddress, FALSE, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCDatBCnt( int connectionId )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    int iRes = pChannel->GetTableCount(FALSE);

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCDatBDatensatz( int connectionId, int deviceAddress, unsigned char *name, int *fieldCount )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!((deviceAddress >= 0) && (deviceAddress < MAX_DATENSATZ_BESCHREIBUNG)) ||
        (name == NULL) || (fieldCount == NULL) )
    {
        pChannel->logEnd("\t\t\t(0)");
        return 0;
    }

    unsigned char szTableName[MAX_NAME];
    unsigned char ucFieldCount;

    int iRes = pChannel->GetTableRecord(deviceAddress, szTableName, &ucFieldCount);

    if (iRes)
    {
        strcpy((char*)name, (const char*)szTableName);
        *fieldCount = ucFieldCount;
    }

    pChannel->logEnd("\t%s\t%i\t(%i)", name, *fieldCount, iRes);
    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCDatBFeld( int connectionId, int dataBNumber, int number, unsigned char *name, int *type, int *size )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, dataBNumber, number);

    // zulaessige Bereiche pruefen
    if (!((dataBNumber >= 0) && (dataBNumber < MAX_DATENSATZ_BESCHREIBUNG)) ||
        !((number >= 0) && (number < MAX_FELD)) ||
        (name == NULL) || (type == NULL) || (size == NULL) )
    {
        pChannel->logEnd("\t\t\t\t(0)");
        return 0;
    }

    unsigned char szFieldName[MAX_NAME];
    unsigned char ucType, ucLength;

    int iRes = pChannel->GetTableField(dataBNumber, number, szFieldName,
        &ucType, &ucLength, FALSE);

    if (iRes)
    {
        strcpy((char*)name, (const char*)szFieldName);
        *type = ucType;
        *size = ucLength;
    }

    pChannel->logEnd("\t%s\t%i\t%i\t(%i)", name, *type, *size, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComCollectData( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return -1;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
        return -1;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, -1);
        return -1;
    }

    int iRes = pChannel->collectData(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == -1) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);
    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComGetDatensatz( int connectionId, unsigned char *buffer, int *bufferLength, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return -1;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    // zulaessige Bereiche pruefen
    if ((buffer == NULL) || (bufferLength == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
        return -1;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, -1);
        return -1;
    }

    int iRes = pChannel->getDatensatz(buffer, bufferLength, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == -1) pChannel->resetWithError(*errorNumber);

    switch ( iRes )
    {
    case 1:
    case 2: {

        // Daten in Puffer uebergeben
        unsigned char ucTmp[MAX_MB_PAKET_DATA * 3 + 1];
        int i;

        for (i=0; i < *bufferLength; i++)
        {
            sprintf((char*)&ucTmp[i * 3], "%02X ", (unsigned char)buffer[i]);
        }
        ucTmp[i * 3] = 0;

        pChannel->logAppend("\t%s\t%i", ucTmp, *bufferLength);
        break;}

    default:
        pChannel->logAppend("\t%s\t%i", "", 0);
    }

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);
    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCReadRecord( int connectionId, int deviceAddress, char *buffer, int *bufferLength, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return -1;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) 
        || (buffer == NULL) || (bufferLength == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
        return -1;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t\t\t%i\t(%i)", *errorNumber, -1);
        return -1;
    }

    int res = pChannel->readRecord(deviceAddress, buffer, bufferLength, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (res == -1)
    {
        pChannel->resetWithError(*errorNumber);

        pChannel->logAppend("\t\t");
    }
    else if (res == 0)
    {
        pChannel->logAppend("\t\t");
    }
    else
    {
        // Daten fuer Log in Puffer uebergeben
        unsigned char ucTmp[MAX_MB_PAKET_DATA * 3 + 1];
        int i;
        for (i=0; i < *bufferLength; i++)
        {
            sprintf((char*)&ucTmp[i * 3], "%02X ", (unsigned char)buffer[i]);
        }
        ucTmp[i * 3] = 0;

        pChannel->logAppend("\t%s\t%i", ucTmp, *bufferLength);
    }
    pChannel->logEnd("\t%i\t(%i)", *errorNumber, res);
    return res;
}

DF_API_EXTERN int DF_API_CALL DFCQuitRecord( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return -1;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) 
        || (errorNumber == NULL))
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
        return -1;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, -1);
        return -1;
    }

    int res = pChannel->quitRecord(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (res == -1) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, res);
    return res;
}

DF_API_EXTERN int DF_API_CALL DFCComClearData( int connectionId, int deviceAddress )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) )
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes = pChannel->clearData(deviceAddress, &iError);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(iError);

    pChannel->logEnd("\t(%i)", iRes);
    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCRestoreRecords( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL))
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(0)", IV_ERROR_FUNCTION_PARAMETER);
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(0)", *errorNumber);
        return 0;
    }

    *errorNumber = IV_ERROR_SUCCESS;
    int iRes = pChannel->restoreRecords(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);
    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCCheckData( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return -1;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
        return -1;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, -1);
        return -1;
    }

    int iRes = pChannel->checkData(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == -1) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}
