/**
 * @file  AEBuffer.h
 * @brief Schnittstelle fuer die Klasse CAEBuffer.
 */
//////////////////////////////////////////////////////////////////////

#ifndef AEBUFFER_H
#define AEBUFFER_H

// defines fuer Setup
#include "aes.h"

// Geraetedateiarchiv einbinden
#include "../File/filedevicezip.h"

// Studiodatei einbinden
#include "../File/filestudio.h"

#pragma pack(1)

// Datensatzgroesse fuer Zutrittslisten (Version 1) definieren.
#define ENTRANCE_RECORD_SIZE_0    22
#define ENTRANCE_RECORD_SIZE_1    35
#define ENTRANCE_RECORD_SIZE_2    19
#define ENTRANCE_RECORD_SIZE_3    16

typedef struct structZeitZone
{
    unsigned char day;
    unsigned short start;
    unsigned short ende;
    unsigned char aktiv;

} STRUCTZeitZone;

typedef struct structZeitModell
{
    unsigned char zeitmodell;
    unsigned short zone;

} STRUCTZeitModell;

typedef struct structPersonalStamm
{
    unsigned char pNr[14];
    unsigned char zeitmodell;
    unsigned char prio;
    
} STRUCTPersonalStamm;

typedef struct structFeiertag
{
    unsigned char zeitmodell;
    unsigned int datum;
    unsigned char prio;

} STRUCTFeiertag;

#pragma pack()

class CAEBuffer
{
public:
    CAEBuffer();
    virtual ~CAEBuffer();

    void ChangeOrderLong(unsigned char *Buf);
    unsigned int ChangeOrderULong(unsigned int val);
    void ULongToBinMSBFirst(unsigned char *Buf,unsigned int Val);
    void ChangeOrderInt(unsigned char *Buf);

protected:
    unsigned int StartDatensatzB, EndDatensatzB;
    int MaxListB;
    int MaxDatB;

    StructDatensatzBeschreibung DatensatzBeschreibung[MAX_DATENSATZ_BESCHREIBUNG];
    StructDatensatzFeld DatensatzFeld[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD];

    StructDatensatzBeschreibung ListenBeschreibung[MAX_DATENSATZ_BESCHREIBUNG];
    StructDatensatzFeld ListenFeld[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD];

public:
    void reset();
    unsigned int GetMDEEKFlashSize_0400();
    unsigned int GetMDEEKFlashSize();
    unsigned int GetEntranceEKFlashSize();
    unsigned int GetEntrance2EKFlashSize();
    void AdjustExtendedMemPointer();
    void PrepareEntrance2EK(unsigned int offs, unsigned char *buf);
    unsigned int GetFlashSetupSize();


    // Speicher fuer Listendaten
    // Listendaten werden über die Channel4Buffer - Klasse verwaltet.
    
    // Sortieren der Datensaetze ueber ASCII - Feld. Offs ist Offset in Datensatz auf Feldbeginn.
    // Allgemein gültiger Sortieralgorithmus
    void QuicksortList(unsigned char *pucMem, unsigned char ucRecordSize, unsigned char ucOffs, int lLeftRecordNum, int lRightRecordNum);
    int QuicksortListPartition(unsigned char *pucMem, unsigned char ucRecordSize, unsigned char ucOffs, int lLeftRecordNum, int lRightRecordNum);
    void QuicksortListSwap(unsigned char *pucMem, unsigned char ucRecordSize, int lLeftRecordNum, int lRightRecordNum);

    static bool comparing(char*, char*);
    void SortList(unsigned char *pucMem, unsigned char ucRecordSize, unsigned char ucOffs);

    // Speicher fuer Zutrittskontrolle (Version 1) - Listendaten
    unsigned int m_ulEntranceListSize[4];
    unsigned int m_ulCompressedListSize[4];
    unsigned char m_ucEntranceRecordSize[4];
    unsigned char *m_pucEntranceListMem[4];
    unsigned char m_pucCompressedListMem[4][FLASH_SIZE_LIST + 1];

    void ClearEntranceListBuffer(int iListNum = -1);
    int MakeEntranceList(int ListNum, int ListCnt, int ListSize, unsigned char *Buf);
    
    // fuer Setupdaten
    FileStudio m_file;

    int GetTableCount(unsigned char ucList = 0);
    
    int GetTableRecord(unsigned char ucNum,unsigned char *szName, 
        unsigned char *pucFieldCount, unsigned char *pucVariable = NULL, 
        unsigned char ucList = 0);

    int GetTableField(unsigned char ucTableNum, unsigned char ucFieldNum, 
        unsigned char *szName, unsigned char *pucType, 
        unsigned char *pucLength, unsigned char ucList = 0);
    
    void DefaultIOModule(StructIOModule *pIOModule);

    // Dateiarchiv fuer Geraetedaten (Firmware) verwalten
    FileDeviceZip m_fileDevice;

protected:
    void CpyFromKonfigBuf(unsigned char *Des,unsigned char *Src,unsigned int Add,int Cnt);
    int GetTableRecordEntrance(unsigned char ucNum,unsigned char *szName, 
        unsigned char *pucFieldCount);
    int GetTableFieldEntrance(unsigned char ucTableNum, unsigned char ucFieldNum, 
        unsigned char *szName, unsigned char *pucType, 
        unsigned char *pucLength);  
};

#endif
