/*###########################################################*/
/** \file aescrypt.h
Autor                 : Bernd Ottmann             01/2012
*/  
/*###########################################################*/

#ifndef _AESCRYPT_H_
#define _AESCRYPT_H_

// Klasse zur Verschluesselung von Daten mittels AES.
class AESCrypt
{
public:
  AESCrypt()
  {
    useInitVectors_AES = 1;
  }
private:
  static const unsigned char sBox[256];
  static const unsigned char InvsBox[256];
  static const unsigned char FFpow[256];
  static const unsigned char FFlog[256];
  static const unsigned char CRC_TAB[256];
  static const unsigned char AES_START_CRC_VALUE = 0x53;
  unsigned char defaultKey_AES[16]; ///< Schluessel für Verschluesselung
  unsigned char initVectorEncrypt_AES[16]; ///< Speicher fuer Initialisierungs Vektor
  unsigned char initVectorDecrypt_AES[16]; ///< Speicher fuer Initialisierungs Vektor
  unsigned char useInitVectors_AES;  ///< benutze Initialisierungs Vektor beim Verschluesseln
  unsigned char roundKeys_AES[176]; ///< Speicher fuer die einzelnen Round Keys

public:
  // High Level Funktionen fuer die Verschluesselung
  void SetKey(unsigned char *key_ptr, unsigned char size);
  unsigned short Encrypt(unsigned char *data, unsigned short raw_size); // Verschluesselung von Nutzdaten
  unsigned int EncryptStream(unsigned char *data, unsigned int raw_size); // Verschluesselung von Nutzdaten
  unsigned char Decrypt(unsigned char *data, unsigned short *size);    // Entschluesselung von Nutzdaten
  unsigned char DecryptStream(unsigned char *data, unsigned int *size);    // Entschluesselung von Nutzdaten

private:
  // Checksumme fuer Verschluesselungsbereich berechnen
  unsigned char CRCCheckData(unsigned char crc_previous, unsigned int size, unsigned char *data);
  // Blockweise
  void EncryptBlock16(unsigned char *block); // Verschluesselung von 16Byte Nutzdaten
  void DecryptBlock16(unsigned char *block); // Entschluesselung von 16Byte Nutzdaten
  // Helper, interne Berechnungsfunktionen
  void ExpandKey(unsigned char *input_key_ptr);
  void Xor(unsigned char *state, unsigned char *round_key, unsigned char size = 16);
  void ShiftRows(unsigned char *block);
  void InvShiftRows(unsigned char *block);
  unsigned char ff_mul(unsigned char a, unsigned char b);
  void MixColumn(unsigned char *block);
  void InvMixColumn(unsigned char *block);
};

#endif
